
/* @(#)midi.c	 91/05/25
 * make sure you've installed Tim Thompson's midi device driver 
 * use the vpixcnf program through sysadmsh to define /dev/midi device
 * 
 * compile: cc -c midi.c
 * link: mv midi.o /usr/vpix/conf/midi
 * 	 /usr/vpix/conf/ectbuild
 */

#include "gdi.h"
#define VPIX 	1
#include	<fcntl.h>

#define EXTERN	extern
#define VOID	void	
#define BYTE	char	
#define WORD	int	
#define ERR	1
#define OK	0
#define TRUE	1
#define FALSE	0

typedef int	(*PFI)();

int	    _fd = -1;
char	    _midi_path[80];
char	_vmidi_path[128];
BYTE	midi_open = FALSE;
int	irqval; 		/* irq from the cnf file */
int	base;			/* base i/o port from cnf file */

int	midimd_init();
int	midimd_close();
int	midimd_release();
int	midimd_acquire();

EXTERN	VOID	gdi_int_set();            /* gdi interface routines */
EXTERN	VOID	gdi_int_clear();            
EXTERN	int	gdi_close_func();
EXTERN	int	gdi_port_enable();
EXTERN	int	gdi_port_disable(); 
EXTERN	PFI	gdi_int_reg_func();
EXTERN	PFI	gdi_int_eoi_func();
EXTERN	char	*gdi_cnf_path();
EXTERN	int	gdi_vterm_func(); 

int	midimd_init();            
int	midimd_release();
int	midimd_acquire(); 

VOID	init_midi();	/* start up */
int	close_midi();	/* shut down */
int	read_midi();	/* pass kernel's signal to V86 program */
int	clear_pseudo_int();	

VOID	init_midi( gdi_dev )
struct gdevice *gdi_dev;
{
	char	*path;

	/* check vpix.cnf entry first */
	path = gdi_cnf_path( "MIDI" );
	/* get the irq and io address from vpix.cnf */
	irqval = gdi_dev->gdi_irqval;
	base = gdi_dev->gdi_iorange->base;
	
	/* use default path in etc/vpixdevs file if none is 
	   specified in users vpix.cnf file */
	if( !path )
	  path = gdi_dev->gdi_dname;  

	if( path )
	{
		strcpy ( _vmidi_path, path );
		gdi_close_func( close_midi );
		gdi_port_enable(base,2);
		gdi_int_reg_func( irqval, read_midi);      
		gdi_int_eoi_func( irqval, clear_pseudo_int);   

		if( midimd_init( path ) != ERR ) {
			midi_open = TRUE;
		}
		else 
			midi_open = FALSE;

	}
}

int read_midi()
{
	gdi_int_set(irqval);   
	return(OK);
}

int clear_pseudo_int()
{
	gdi_int_clear(irqval);  
	return(OK);
}

int	close_midi()
{
	return( midimd_close() );
}

int	midimd_init( path )
	char	*path;
{
	strcpy (_midi_path, path);
	return (midimd_acquire ());
}

int midimd_acquire()
{

        _fd = open(_midi_path,O_RDWR | O_EXCL);
	return( _fd == -1 ? ERR : OK );
}


int	midimd_close()
{
	return (midimd_release ());
}

int midimd_release()
{
	int	rc = ERR;

	if	( _fd != -1 )
	{
		gdi_port_disable(base,2);
		rc = close( _fd );
		_fd = -1;
	}
	return( rc );
}

